import { html } from '@codemirror/lang-html';
import { LRLanguage, LanguageSupport } from '@codemirror/language';
import { parseMixed } from '@lezer/common';
import { parser as parser$1 } from '@lezer/html';
import { ExternalTokenizer, LRParser } from '@lezer/lr';
import { styleTags, tags } from '@lezer/highlight';

// This file was generated by lezer-generator. You probably shouldn't edit it.
const shortCommentContent$1 = 56,
  longCommentContent$1 = 57,
  htmlCommentContent$1 = 58,
  templateTagContent$1 = 59,
  MoustacheExpression = 9;

// Ascii values
const parenOpen = 40;
const parenClose = 41;
const squareOpen = 91;
const squareClose = 93;
const curlyOpen = 123;
const curlyClose = 125;
const colon = 58;
const hash = 35;
const at = 64;
const slash = 47;
const greaterThan = 62;
const dash = 45;
const quoteDouble = 34;
const quoteSingle = 39;
const backslash = 92;
const newline = 10;
const asterisk = 42;
const tick = 96;
const prefixes = [colon, hash, at, slash];
const shortCommentEnd = [curlyClose, curlyClose];
const longCommentEnd = [dash, dash, curlyClose, curlyClose];
const htmlCommentEnd = [dash, dash, greaterThan];
// Is there a way to make this a build time (rather than eval-time?) operation?
const closingTemplateTag = '</template>'.split('').map((char) => char.charCodeAt(0));
function matchForComment(commentEndPattern, commentToken, input) {
    for (let found = 0, i = 0;; i++) {
        if (input.next < 0) {
            if (i)
                input.acceptToken(commentToken);
            break;
        }
        let hasMatch = commentEndPattern[found] === input.next;
        if (!hasMatch) {
            found = 0;
            input.advance();
            break;
        }
        if (found === commentEndPattern.length - 1) {
            if (i > commentEndPattern.length - 1) {
                input.acceptToken(commentToken, 1 - commentEndPattern.length);
                break;
            }
            else {
                console.warn('Reached end of comment but there is still content left');
            }
            break;
        }
        found++;
        input.advance();
    }
}
const shortCommentContent = new ExternalTokenizer((input) => {
    return matchForComment(shortCommentEnd, shortCommentContent$1, input);
});
const longCommentContent = new ExternalTokenizer((input) => {
    return matchForComment(longCommentEnd, longCommentContent$1, input);
});
const htmlCommentContent = new ExternalTokenizer((input) => {
    return matchForComment(htmlCommentEnd, htmlCommentContent$1, input);
});
const templateTagContent = new ExternalTokenizer((input) => {
    return matchForComment(closingTemplateTag, templateTagContent$1, input);
});
// TODO: string handler does not handle interpolation
function createStringHandler(input) {
    let inString = false;
    let inStringType = null;
    let inStringIgnoreNext = false;
    return () => {
        if (inString) {
            if (inStringIgnoreNext) {
                inStringIgnoreNext = false;
                return true;
            }
            if (input.next === backslash) {
                inStringIgnoreNext = true;
                return true;
            }
            if (inStringType === 'double' && input.next === quoteDouble) {
                inString = false;
                inStringType = null;
                return true;
            }
            if (inStringType === 'single' && input.next === quoteSingle) {
                inString = false;
                inStringType = null;
                return true;
            }
            if (inStringType === 'template' && input.next === tick) {
                inString = false;
                inStringType = null;
                return true;
            }
            return true;
        }
        if (input.next === quoteDouble) {
            inString = true;
            inStringType = 'double';
            return true;
        }
        if (input.next === quoteSingle) {
            inString = true;
            inStringType = 'single';
            return true;
        }
        if (input.next === tick) {
            inString = true;
            inStringType = 'template';
            return true;
        }
        return false;
    };
}
function createCommentHandler(input) {
    let inLineComment = false;
    let inBlockComment = false;
    return () => {
        if (inLineComment) {
            if (input.next === newline) {
                inLineComment = false;
                return true;
            }
            return true;
        }
        if (inBlockComment) {
            if (input.next === asterisk && input.peek(1) === slash) {
                inBlockComment = false;
                return true;
            }
            return true;
        }
        if (input.next === slash && input.peek(1) === slash) {
            inLineComment = true;
            return true;
        }
        if (input.next === slash && input.peek(1) === asterisk) {
            inBlockComment = true;
            return true;
        }
        return false;
    };
}
// closes on a delimiter that probably isn't in the expression
new ExternalTokenizer((input) => {
    if (prefixes.includes(input.next)) {
        return;
    }
    const commentHandler = createCommentHandler(input);
    const stringHandler = createStringHandler(input);
    let stack = [];
    const popIfMatch = (match) => {
        const idx = stack.lastIndexOf(match);
        if (idx !== -1) {
            while (stack.length > idx) {
                stack.pop();
            }
        }
    };
    for (let pos = 0;; pos++) {
        // end of input
        if (input.next < 0) {
            if (pos > 0)
                input.acceptToken(MoustacheExpression);
            break;
        }
        if (commentHandler() || stringHandler()) {
            input.advance();
            continue;
        }
        if (stack.length === 0 &&
            (input.next === curlyClose || input.next === parenClose || input.next === squareClose)) {
            input.acceptToken(MoustacheExpression);
            break;
        }
        switch (input.next) {
            case parenOpen:
                stack.push('(');
                break;
            case parenClose:
                popIfMatch('(');
                break;
            case squareOpen:
                stack.push('[');
                break;
            case squareClose:
                popIfMatch('[');
                break;
            case curlyOpen:
                stack.push('{');
                break;
            case curlyClose:
                popIfMatch('{');
                break;
        }
        input.advance();
    }
});

const glimmerHighlighting = styleTags({
    'Text RawText': tags.content,
    'StartTag StartCloseTag SelfClosingEndTag EndTag': tags.angleBracket,
    TagName: tags.tagName,
    'MismatchedCloseTag/TagName': [tags.tagName, tags.invalid],
    AttributeName: tags.attributeName,
    // Glimmer
    TemplateTag: tags.definition(tags.annotation),
    '{{ }}': tags.bracket,
    '( )': tags.paren,
    Function: tags.function(tags.name),
    Splattributes: tags.derefOperator,
    Comment: tags.blockComment,
    ShortComment: tags.blockComment,
    LongComment: tags.blockComment,
    HtmlComment: tags.blockComment,
    ArgName: tags.attributeName,
    Argument: tags.propertyName,
    NamedBlock: tags.tagName,
    Is: tags.definitionOperator,
    Pipe: tags.definitionOperator,
    as: tags.definitionOperator,
    let: tags.function(tags.definitionOperator),
    this: tags.self,
    yield: tags.operatorKeyword,
    outlet: tags.operatorKeyword,
    component: tags.function(tags.definitionKeyword),
    modifier: tags.function(tags.definitionKeyword),
    helper: tags.function(tags.definitionKeyword),
    hash: tags.function(tags.definitionKeyword),
    array: tags.function(tags.definitionKeyword),
    on: tags.function(tags.definitionKeyword),
    concat: tags.function(tags.operatorKeyword),
    'if then catch unless else each each-in': tags.controlKeyword,
    // TODO:
    UnquotedAttributeValue: tags.attributeValue,
    'DoubleQuote SingleQuote AttributeValueContent': tags.attributeValue,
    'EntityReference CharacterReference': tags.character,
    '[ ]': tags.squareBracket,
    ComponentName: tags.className,
    StyleAttributeName: tags.propertyName,
    BlockType: tags.controlKeyword,
    BlockPrefix: tags.typeOperator,
    'UnknownBlock/BlockType': tags.invalid,
    UnknownBlockContent: tags.invalid,
    Variable: tags.variableName,
    Modifier: tags.modifier,
    ModifierValue: tags.function(tags.modifier),
});

// This file was generated by lezer-generator. You probably shouldn't edit it.
const spec_identifier = {__proto__:null,this:36, as:38, if:40, unless:42, else:44, let:46, and:48, or:50, not:52, eq:54, neq:56, "not-eq":58, gt:60, gte:62, lt:64, lte:66, array:68, concat:70, hash:72, each:74, "each-in":76, on:78, modifier:80, component:82, helper:84, "in-element":86, yield:88, outlet:90};
const parser = LRParser.deserialize({
  version: 14,
  states: "(dQVQ`OOOwQhO'#C^O!PQaO'#CbO!XQbO'#CcO!aQ`O'#CmO$oOpO'#CeO$vO`O'#CeO${QdO'#CdOOQO'#Ca'#CaOOQO'#D]'#D]O%^Q`O'#D^OOQO'#Dl'#DlOOQO'#D_'#D_QVQ`OOOOQW'#D`'#D`O%cQhO,58xOOQO,58x,58xOOQP'#Da'#DaO%kQaO,58|OOQO,58|,58|OOQQ'#Db'#DbO%sQbO,58}OOQO,58},58}OOQO,59X,59XO(RQ`O'#ChOOOO'#Cg'#CgOOOO'#Dd'#DdO(]OpO,59POOQS,59P,59POOOO'#Dt'#DtO(dOpO,59POOQS'#Dc'#DcO(iQdO,59OOOQO,59O,59OOOQO,59x,59xOOQO-E7]-E7]OOQW-E7^-E7^OOQO1G.d1G.dOOQP-E7_-E7_OOQO1G.h1G.hOOQQ-E7`-E7`OOQO1G.i1G.iOOOO,59S,59SO(zQ`O,59SO)VQ`O,59SOOOO-E7b-E7bOOQS1G.k1G.kOOQS-E7a-E7aOOQO1G.j1G.jO!fQ`O1G.nOOOO1G.n1G.nO)_Q`O1G.nO)dQ`O7+$YO!fQ`O7+$YOOOO<<Gt<<GtO)iQ`O<<GtOOOOAN=`AN=`",
  stateData: ")p~O!_OS~ORPOYTO!aQO!cRO!eVO!gSO!iTO!jUO!lXO!mYO~OS`O!]^O~O!YaO!bcO~O!ZdO!dfO~O!fgO~O]hO^iO`iObmOcmOdmOemOfmOgmOhmOimOjmOkmOlmOmmOnmOomOpmOqmOrmOsmOtmOumOvmOwmOxmOymOzmO{mO|mO}mO!fiO!gSO~O!OlO~P!fO!fnO~OYTO![oO!iTO!jUO!kqO~O!frO~OSuO!]^O~O!YaO!bwO~O!ZdO!dyO~O]hO^iO`iObmOcmOdmOemOfmOgmOhmOimOjmOkmOlmOmmOnmOomOpmOqmOrmOsmOtmOumOvmOwmOxmOymOzmO{mO|mO}mO!gSO~O_zO!f{O~P%{O!O!OO~P!fO!O!OO~OYTO![oO!iTO!jUO!k!QO~O^!RO_ZX!fZX~O_!SO!f!TO~O^!VO~O_!WO~O_!YO~O!_~",
  goto: "$e!iPP!jPP!j!n!n!n!rP!z#XPPPP#`PPPPPPPPPPPPPPPPPPPPPPPPPPPPP!j!j#k#q#w#}$T$ZPPPPPPP$aPPPPPPP#XTZO]TWO]SZO]ToVpSjTkQ|hQ!U!RR!X!VZiThk!R!VSZO]ZiThk!R!VQ]ORs]Q_PRt_QbQRvbQeRRxeQpVR!PpQkTR}kT[O]",
  nodeNames: "⚠ Template GlimmerTemplateTag TemplateTag TemplateTag Comment ShortComment LongComment HtmlComment MoustacheExpression {{ SExpression CallExpression ( Is ) Pipe Argument this as if unless else let and or not eq neq not-eq gt gte lt lte array concat hash each each-in on modifier component helper in-element yield outlet }} Splattributes NamedBlock",
  maxTerm: 75,
  nodeProps: [
    ["closedBy", 3,"templateTagEnd",10,"}}",13,"("],
    ["openedBy", 4,"templateTagStart",15,")",46,"{{"]
  ],
  propSources: [glimmerHighlighting],
  skippedNodes: [0],
  repeatNodeCount: 6,
  tokenData: "+W~RhX^!mpq!mxy#eyz#j|}!m}!O#o!O!P$`![!]&P!^!_&U!_!`)X!b!c)^!c!})c#T#o)c#o#p)q#p#q*t#q#r*y#y#z!m$f$g!m#BY#BZ!m$IS$I_!m$I|$JO!m$JT$JU!m$KV$KW!m&FU&FV!m~!rZ!_~X^!mpq!m|}!m#y#z!m$f$g!m#BY#BZ!m$IS$I_!m$I|$JO!m$JT$JU!m$KV$KW!m&FU&FV!m~#jO]~~#oO_~~#rP}!O#u~#xQ!`!a$O#q#r$T~$TO!k~~$WP#q#r$Z~$`O!d~~$cP!O!P$f~$iP!O!P$l~$oP#T#U$r~$uP#h#i$x~${P#h#i%O~%RP#f#g%U~%XP#]#^%[~%_P#U#V%b~%eP#i#j%h~%kP#h#i%n~%qP#X#Y%t~%wP#g#h%z~&PO!l~~&UO!m~~&XRqr&b!P!Q&s#h#i(Q~&eP}!O&h~&kP}!O&n~&sO!e~~&vP#h#i&y~&|P#X#Y'P~'SP#a#b'V~'YP#d#e']~'`P#`#a'c~'fP#T#U'i~'lP#h#i'o~'rP#X#Y'u~'xP!`!a'{~(QOS~~(TP#X#Y(W~(ZP#a#b(^~(aP#d#e(d~(gP#`#a(j~(mP#T#U(p~(sP#h#i(v~(yP#X#Y(|~)PP!`!a)S~)XOR~~)^O^~~)cO!g~~)hR!f~!Q![)c!c!})c#T#o)c~)tP#o#p)w~)|RY~qr*Vst*j!P!Q*o~*[P!a~}!O*_~*bP}!O*e~*jO!c~~*oO!i~~*tO!j~~*yO`~R*|P#q#r+PR+WO!OQ!bP",
  tokenizers: [shortCommentContent, longCommentContent, htmlCommentContent, templateTagContent, 0, 1],
  topRules: {"Template":[0,1]},
  specialized: [{term: 68, get: value => spec_identifier[value] || -1}],
  tokenPrec: 398
});

// import type { SyntaxNode } from '@lezer/common';
const glimmerParser = parser;
const glimmerLanguage = LRLanguage.define({
    parser: parser.configure({
        wrap: parseMixed((node) => {
            let { name } = node.type;
            switch (name) {
                case 'Template':
                    return null;
                case 'Comment':
                case 'MoustacheExpression':
                case 'NamedBlock':
                case 'Splattributes':
                case 'Argument':
                    return { parser };
            }
            return { parser: parser$1 };
        }),
        props: [
        // indentNodeProp.add({
        //   Element: (context) => {
        //     let after = /^(\s*)(<\/)?/.exec(context.textAfter);
        //     if (context.node.to <= context.pos + (after?.[0].length || 0)) return context.continue();
        //     return context.lineIndent(context.node.from) + (after?.[2] ? 0 : context.unit);
        //   },
        //   Block: (context) => {
        //     const node = context.node;
        //     const text = context.textAfter.trim();
        //     if (text.startsWith('{/')) {
        //       const name = node.name;
        //       if (
        //         (name === 'IfBlock' && text.startsWith('{/if')) ||
        //         (name === 'EachBlock' && text.startsWith('{/each')) ||
        //         (name === 'KeyBlock' && text.startsWith('{/key'))
        //       ) {
        //         return context.lineIndent(context.node.from);
        //       }
        //       return null;
        //     }
        //     if (node.name === 'IfBlock' || node.name === 'EachBlock') {
        //       if (text.startsWith('{{:else')) return context.lineIndent(node.from);
        //       if (text.startsWith('{{else')) return context.lineIndent(node.from);
        //     }
        //     // not sure if this needed to be duplicated
        //     let after = /^(\s*)(<\/)?/.exec(context.textAfter);
        //     if (context.node.to <= context.pos + (after?.[0].length || 0)) return context.continue();
        //     return context.lineIndent(context.node.from) + (after?.[2] ? 0 : context.unit);
        //   },
        //   'BlockOpen BlockClose BlockInline': (context) => {
        //     return context.column(context.node.from) + context.unit;
        //   },
        //   'OpenTag CloseTag SelfClosingTag': (context) => {
        //     return context.column(context.node.from) + context.unit;
        //   },
        //   Document: (context) => {
        //     if (context.pos + (/\s*/.exec(context.textAfter)?.[0].length || 0) < context.node.to) {
        //       return context.continue();
        //     }
        //     let endElt: SyntaxNode | null = null;
        //     let close: SyntaxNode;
        //     for (let cur = context.node; ; ) {
        //       let last = cur.lastChild;
        //       if (!last || last.name != 'Element' || last.to != cur.to) break;
        //       endElt = cur = last;
        //     }
        //     if (
        //       endElt?.lastChild &&
        //       !(
        //         (close = endElt.lastChild) &&
        //         (close.name === 'CloseTag' || close.name === 'SelfClosingTag')
        //       )
        //     ) {
        //       return context.lineIndent(endElt.from) + context.unit;
        //     }
        //     return null;
        //   },
        // }),
        // foldNodeProp.add({
        //   Block: (node) => {
        //     const open = `${node.name}Open`;
        //     const close = `${node.name}Close`;
        //     const first = node.firstChild;
        //     const last = node.lastChild;
        //     if (!first || first.name !== open) return null;
        //     return {
        //       from: first.to,
        //       to: last?.name === close ? last.from : node.to,
        //     };
        //   },
        //   Element: (node) => {
        //     let first = node.firstChild;
        //     let last = node.lastChild;
        //     if (!first || first.name != 'OpenTag') return null;
        //     return {
        //       from: first.to,
        //       to: last?.name === 'CloseTag' ? last.from : node.to,
        //     };
        //   },
        // }),
        ],
    }),
    // languageData: {
    // commentTokens: {
    //   block: { open: '<!--', close: '-->' },
    //   devBlock: { open: '{{!--', close: '--}}' },
    //   devBlockShort: { open: '{{!', close: '}}' },
    // },
    // indentOnInput: /^\s*((<\/\w+\W)|(\{:(else|then|catch))|(\{\/(if|unless|each|await|key)))$/,
    // wordChars: '-._',
    // autocomplete: htmlCompletionSource,
    // },
});
function glimmer() {
    return new LanguageSupport(glimmerLanguage, [html().support]);
}

export { glimmer, glimmerLanguage, glimmerParser };
